/* this file stores the functions to open connections
 * to the kismet servers and grab information from them.
 */
 
 
 /*
  * Open socket to kismet server and wait for initialization info.
  * input:  empty file descriptor
  * 	    the file descriptor returns with the assigned number
  *			the access mode is file I/O non blocking
  * output: 1 if connection OK
  * 		0 if connection FAILED
  */
  
#define MAXPACKETLIST 500
#include "connector_kismet.h"
  
int open_kismet_client(FILE** client_fd,int& client_socket,short int in_port, char *in_host)
{
	
	char errstr[1024];
    char status[STATUS_MAX];

    // Active server
    int sv_valid;

    // Server info
    short int port;
    char hostname[MAXHOSTNAMELEN];



    struct sockaddr_in client_sock, local_sock;
    struct hostent *client_host;

    // Timestamp
    time_t serv_time;
    
    char major[24], minor[24], tiny[24];
    char build[24];
    
	time_t start_time;

    char servername[32];

    port = in_port;

    // Resolve the hostname we were given/found to see if it's actually
    // valid
    if ((client_host = gethostbyname(in_host)) == NULL) {
        snprintf(errstr, 1024, "TcpClient could not resolve host \"%s\"\n", in_host);
        return (-1);
    }

    strncpy(hostname, in_host, MAXHOSTNAMELEN);

    // Set up our socket
    
    memset(&client_sock, 0, sizeof(client_sock));
    client_sock.sin_family = client_host->h_addrtype;
    memcpy((char *) &client_sock.sin_addr.s_addr,
		client_host->h_addr_list[0],
        client_host->h_length);
    client_sock.sin_port = htons(in_port);

    if ((client_socket = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        snprintf(errstr, 1024, "TcpClient socket() failed %d (%s)\n",
                 errno, strerror(errno));
        return (-2);
    }

    // Bind to the local half of the pair
    local_sock.sin_family = AF_INET;
    local_sock.sin_addr.s_addr = htonl(INADDR_ANY);
    local_sock.sin_port = htons(0);

    if (bind(client_socket, (struct sockaddr *) &local_sock, sizeof(local_sock)) < 0) {
        snprintf(errstr, 1024, "FATAL: TcpClient bind() failed %d (%s)\n",
                 errno, strerror(errno));
        return (-3);
    }

    // Connect
    if (connect(client_socket, (struct sockaddr *) &client_sock, sizeof(client_sock)) < 0) {
        snprintf(errstr, 1024, "FATAL: TcpClient connect() failed %d (%s)\n",
                 errno, strerror(errno));
        return (-4);
    }

	// get connection flags
	
    int save_mode = fcntl(client_socket, F_GETFL, 0);
    if (save_mode == -1) {
        snprintf(errstr, 1024, "FATAL:  TcpClient connect() failed fcntl get %d (%s)\n",
                 errno, strerror(errno));
        return (-5);
    }
	
	// set non-blocking mode
	
    if (fcntl(client_socket, F_SETFL, save_mode | O_NONBLOCK) < 0) {
        snprintf(errstr, 1024, "FATAL:  TcpClient connect() failed fcntl set %d (%s)\n",
                 errno, strerror(errno));
        return (-6);
    }

    sv_valid = 1;

	// establish a file i/o socket
	
    *client_fd = fdopen(client_socket, "r+");

    return 0;
}

 /*
  * Close socket to kismet server.
  * input: file descriptor
  * output: NONE
  */
  
void close_kismet_client(FILE* filedescriptor)
{
}

 /*
  * Select and process all packets on each call.
  * input:  socket file descriptor
  * 	 	empty structure to load the packets
  * output: qty of packets processed
  * 		0 if no packets
  */
  
int load_kismet_client(char* in_data,vector<packet_info>& packetlist)
{
	
	char header[65];
    char bssid_str[18];
    mac_addr bssid;
    int junkmajor, junkminor, junktiny;

    if (sscanf(in_data, "%64[^:]", header) < 1) {
        return 0;
    }

    unsigned int hdrlen = strlen(header) + 2;
    if (hdrlen >= strlen(in_data))
        return 0;
	
	if (!strncmp(header, "*PACKET", 64)) {
        packet_info packinfo;
        memset(&packinfo, 0, sizeof(packet_info));
        char smac[18], dmac[18], bmac[18];
        short int sip[4], dip[4];
        int sport, dport;
	
	// Added one %d and the (int *) &packinfo.ts.tv_usec, line.
	// Added one (at the end) %d and the (int *) &packinfo.signal, line.
	// Added one (at the end) %d and the (int *) &packinfo.retry, line. 
	// Added one (at the end) %d and the (int *) &packinfo.fullsize, line.
 	// Added one (at the end) %d and the (int *) &packinfo.datarate, line. 			
	// DD @ INRIA
		
        if (sscanf(in_data+hdrlen, "%d %d %ld %ld %d %d %d %17s %17s %17s "
                   "\001%32[^\001]\001 %d %hd.%hd.%hd.%hd %hd.%hd.%hd.%hd %d %d %d "
                   "\001%16[^\001]\001 %d %d %d %d \n",
                   (int *) &packinfo.type,
                   (int *) &packinfo.subtype,
                   (long *) &packinfo.ts.tv_sec,
		   (long *) &packinfo.ts.tv_usec, //Added by DD @ INRIA
                   &packinfo.encrypted, &packinfo.interesting, &packinfo.beacon,
                   smac, dmac, bmac,
                   packinfo.ssid,
                   (int *) &packinfo.proto.type,
                   &sip[0], &sip[1], &sip[2], &sip[3], &dip[0], &dip[1], &dip[2], &dip[3],
                   (int *) &sport, (int *) &dport,
                   (int *) &packinfo.proto.nbtype, packinfo.proto.netbios_source, (int *) &packinfo.signal,
					&packinfo.retry, &packinfo.fullsize, &packinfo.datarate) < 22)
            return 0; 


#ifdef DEBUG
				printf("signal received: %d \n",packinfo.signal);
#endif
		
	    // end of changes
	    
        packinfo.source_mac = smac;
        packinfo.dest_mac = dmac;
        packinfo.bssid_mac = bmac;

        packinfo.proto.sport = sport;
        packinfo.proto.dport = dport;

        for (unsigned int x = 0; x < 4; x++) {
            packinfo.proto.source_ip[x] = sip[x];
            packinfo.proto.dest_ip[x] = dip[x];
        }

        packetlist.push_back(packinfo);

	}
	return 0;
}




/*
 * Send a data line
 * The line must be ended with a \n character.
 * 
 */

int send_kismet_client(int client_socket, char* data)
{
	
	char status[STATUS_MAX];
	char errstr[1024];
	// Clear the status
    memset(status, 0, STATUS_MAX);

    int selected;
    fd_set read_set;
    fd_set write_set;

    FD_ZERO(&read_set);
    FD_SET(client_socket, &read_set);
    FD_ZERO(&write_set);
    FD_SET(client_socket, &write_set);

    struct timeval tim;

    tim.tv_sec = 0;
    tim.tv_usec = 0;

    // Enter the select loop
    if ((selected = select(client_socket+1, &read_set, &write_set, NULL, &tim)) < 0) {
        if (errno != EINTR) {
            snprintf(errstr, 1024, "TcpServer select() returned %d (%s)\n",
                     errno, strerror(errno));

            close(client_socket);
            return (-1);
        }
    }
	
	   if (strlen(data) > 0 && FD_ISSET(client_socket, &write_set)) {
        int res = write(client_socket, data, strlen(data));

        if (res <= 0) {
            if (res == 0 || (errno != EAGAIN && errno != EINTR)) {
                snprintf(errstr, 1024, "Write error on socket (%d): %s", errno,
                         strerror(errno));

                close(client_socket);
                return(-1);
            }
        } else {

        }
    }
	return(0);
}



/*
 * Receive a data line
 * input: file descriptor
 * output: data string
 */

int receive_kismet_client(int client_socket,FILE* client_fd, char* data)
{


	char errstr[1024];
	char status[STATUS_MAX];
	// Clear the status
    memset(status, 0, STATUS_MAX);

    int selected;
    fd_set read_set;
    fd_set write_set;

    FD_ZERO(&read_set);
    FD_SET(client_socket, &read_set);
    FD_ZERO(&write_set);
    FD_SET(client_socket, &write_set);

    struct timeval tim;

    tim.tv_sec = 0;
    tim.tv_usec = 0;

    // Enter the select loop
    if ((selected = select(client_socket+1, &read_set, &write_set, NULL, &tim)) < 0) {
        if (errno != EINTR) {
            snprintf(errstr, 1024, "TcpServer select() returned %d (%s)\n",
                     errno, strerror(errno));

            close(client_socket);
            return (-1);
        }
    }
        // test if we have something to read.	
	   if (!FD_ISSET(client_socket, &read_set))
        return 0;
			

	if (fgets(data, 2048, client_fd) == NULL) {


        if (errno != 0 && errno != EAGAIN) {
            snprintf(errstr, 1024, "Read error %d (%s), closing the connection.",
                     errno, strerror(errno));

            close(client_socket);
            return (-1);
        }

        if (feof(client_fd)) {
            snprintf(errstr, 1024, "socket returned EOF, server has closed the connection.");

            close(client_socket);
            return (-2);
        }


	// empty buffer: receives 0
	 return(0); 
	
	
    }
	
	// trash on buffer
	
    if (strlen(data) < 2)
        return 0;


}


/*
 * check kismet client
 * checks if there data waiting to read at the file descriptor given
 * returns 1 if there is something to read
 * returns -1 if error.
 * returns 0 if no news.
 */


int check_kismet_client(int client_socket)
{

	char errstr[1024];
	char status[STATUS_MAX];
	// Clear the status
    memset(status, 0, STATUS_MAX);

    int selected;
    fd_set read_set;
    fd_set write_set;

    FD_ZERO(&read_set);
    FD_SET(client_socket, &read_set);
    FD_ZERO(&write_set);
    FD_SET(client_socket, &write_set);

    struct timeval tim;

    tim.tv_sec = 0;
    tim.tv_usec = 0;

    // Enter the select loop
    if ((selected = select(client_socket+1, &read_set, &write_set, NULL, &tim)) < 0) {
        if (errno != EINTR) {
            snprintf(errstr, 1024, "TcpServer select() returned %d (%s)\n",
                     errno, strerror(errno));
            //sv_valid = 0;
            close(client_socket);
            return (-1);
        }
    }
        // test if we have something to read.	
	   if (!FD_ISSET(client_socket, &read_set))
        return 0;
	   else
		return 1;
	
	
}
