/***************************************************************************
 *            packet_processor2.cc
 *
 *  Mon Apr  4 10:22:25 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* This program is the packet processor engine. It grabs the packets
  * from the kismet server(s) and processes them to build statistics
  * based on them. It serves this information thru a simple question/
  * answer protocol, listening on a specific socket.
  */
  
  /* Protocol Specification:
   * Question: <PARAMETER> <STATION MAC ADDRESS> <TIME REFERENCE> CR
   * Except in parameter LAST, where there is no <TIME REFERENCE>.
   * Answer: <PARAMETER> <STATION MAC ADDRESS> <QTY> <PAIRS [seconds,parameter]> CR
   * Error: ERROR CR
   * List of parameters:
   * PACKET_TRAFFIC
   * PACKET_TRAFFIC_IN
   * PACKET_TRAFFIC_OUT
   * BIT_TRAFFIC_IN
   * BIT_TRAFFIC_OUT
   * TOTAL_TRAFFIC
   * CONNECTION_SPEED
   * POWER
   * RETRY_TIMES
   * LAST
   * Station mac address example: 0D:0E:0F:0A:0B:0C
   * Quantity is the quantity of data pairs transmitted
   * Pairs are the independent (first member) and dependent (second member)
   * separated by a space between the pairs and with a comma between parameters
   * Every datan package is terminated by a CR character, in order to use fgets.
   * ERROR is returned when:
   * -There is a failure looking for the mac address
   * -...
   */
   
  /* MISSING THINGS TO IMPLEMENT:
   * - capture Signals to turn off server  -done-
   * - logging system: develop a log format
   * - ring buffer with programmable size for all parameters -done-
   * - fcs comparison to check for repeated packets 
   * - (together with the former feature) retry bit check for uniqueness -not feasible-
   * - script to start server before user interface 
   * - configuration via a config file 
   * - implement the configuration of every feature requested
   * - build statistics for every mac address (network and station) -done-   
   */

// for test without need to connect to servers (does not include that code)
//#ifndef NO_SERVER_TEST
//#define NO_SERVER_TEST
///#endif
#ifdef NO_SERVER_TEST
#undef NO_SERVER_TEST
#endif

   
// for sockets support

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

// end of socket support


#include <string>
#include <iostream.h>
#include "configfile.h"
#include <time.h>

#ifdef HAVE_CONFIG_H
//#  include <config.h>
#endif

#define CONFIGFILE "wimon.conf"

//#define DEBUG
//#define DEBUG_R
#ifdef DEBUG
#undef DEBUG
#endif

#include <iostream>
#include <cstdlib>
#include <string>
#include <sstream>
#include <vector>
#include <map>

// includes for the kismet support

#include "packet.h"
#include "tcpclient.h"

// macro to define the configfile

#define CONFIGFILE "wimon.conf"

#include "packet_processor2.h"
#include "connector_kismet.h"

// macros to define the sources.

//half of search duplicate window in microseconds
#define WINDOWHALF 100 

//quantity of bins in memory
#define TRAFFIC_BIN_MAX_SIZE 40

// map that contains all the sources with the corresponding
// bin_infos. <mac_address,info>

map<mac_addr,device_info> all_devices;

void save_packets(map<unsigned long,unsigned int>& traffic,FILE* logfile);

void save_one_bin(map<unsigned long,unsigned int>::iterator i,FILE* logfile);

// we are using the standard c++ templates.
using namespace std;

//is log enabled?
int logfil=0;

/*
 * This function reports the error and
 * exits back to the shell :
 */
static void
bail(const char *on_what) {
    if ( errno != 0 ) {
        fputs(strerror(errno),stderr);
        fputs(": ",stderr);
    }
    fputs(on_what,stderr);
    fputc('\n',stderr);
    exit(1);
}


// the main function: the external parameters are ignored.


int
main (int argc, char *argv[])
{

// socket support 

    int z;
	char *srvr_addr = NULL;
    char *srvr_port = "9099";
    struct sockaddr_in adr_srvr;/* AF_INET */
    struct sockaddr_in adr_clnt;/* AF_INET */
    int len_inet;               /* length  */
    int s;                       /* Socket */
    int c;                /* Client socket */
    int n;                        /* bytes */
    time_t td;        /* Current date&time */
    char dtbuf[128];     /* Date/Time info */

// end of socket support	
	
// data buffer to send or receive

char data[1024];	
	
// file descriptors for communication
FILE *client_fd1;	
FILE *client_fd2;	
	
// file descriptors to log packets
FILE *packlog1;
FILE *packlog2;
FILE *mergepacklog;	

// open sockets

int client_socket1;
int client_socket2;

// to enable socket address reuse option

int so_reuseaddr=1;
	
// packet lists to grab packets from the structures.
// these ones are only intermediate lists to identify the
// source and then merge the info. this processing can
// be simplified without using them	
	
vector<packet_info> listpacket1;
vector<packet_info> listpacket2;

// per second packet qty traffic information

map<unsigned long,unsigned int> traffic_bin;

// mergepacket, map of merged packets. The index key is the
// time in microseconds, calculated from the packet time.
	
map<uint64_t,packet_info> mergepacket;

// count: used to repeat the acquisition for testing purposes.
int count=0;

// packet1 and packet2: original packet list from every source.
vector<packet_info> packet1;
vector<packet_info> packet2;

// logfile map
map<mac_addr,logfile_structure> logfiles;

// last update 
unsigned long int last_update;


// Catch the interrupt handlers
signal (SIGPIPE,SIG_IGN);

// Process config file

char first_server[80];
char second_server[80];
unsigned int port;
char Laddress[80];
unsigned int Lport;



// object used to parse the config file

ConfigFile *conf= new ConfigFile;

// configuration file parsing

if (conf->ParseConfig(CONFIGFILE)<0)
	{
	printf("Error trying to parse the config file");
	exit(1);
	}

if (conf->FetchOpt("FirstServer")!="")
	{
	strcpy(first_server,conf->FetchOpt("FirstServer").c_str());
	}
else
	{	
	exit(1);
	}
	

if (conf->FetchOpt("SecondServer")!="")
	{
	strcpy(second_server,conf->FetchOpt("SecondServer").c_str());
	}
else
	{	
	exit(1);
	}

	
if (conf->FetchOpt("Port")!="")
	{
	sscanf(conf->FetchOpt("Port").c_str(),"%d",&port);
	}
else
	{	
	exit(1);
	}


if (conf->FetchOpt("LAddress")!="")
	{
	strcpy(Laddress,conf->FetchOpt("LAddress").c_str());
	}
else
	{	
	exit(1);
	}

	
if (conf->FetchOpt("LPort")!="")
	{
	sscanf(conf->FetchOpt("LPort").c_str(),"%d",&Lport);
	}
else
	{	
	exit(1);
	}

if (conf->FetchOpt("Log")!="")
	{
	if (strcmp(conf->FetchOpt("Log").c_str(),"yes")==0)
		{
		logfil=1;
		}
	else
		{
		logfil=0;
		}
	}
else
	{	
	exit(1);
	}


// configuration done. now connect to the kismet server
	
vector<string> kismet_servers_strings;
vector<string> :: iterator kismet_servers_it;	
int server_counter;
vector<kismet_server_struct> kismet_servers;
	
kismet_servers_strings=conf->FetchOptVec("KismetSourceServer");	
for (server_counter=0;server_counter<kismet_servers_strings.size();server_counter++)
	{
	kismet_server_struct temp_server_structure;
	//strcpy(temp_server_structure.IP,kismet_servers_strings[server_counter].c_str());
    temp_server_structure.IP=kismet_servers_strings[server_counter];				
	kismet_servers.push_back(temp_server_structure);	
	printf("%s",kismet_servers_strings[server_counter].c_str());
	}

	
#ifdef DEBUG 	
	printf("fs:%s ss:%s port:%d",first_server,second_server,port);
	printf("la:%s lport:%d",Laddress,Lport);
#endif

//now create the logfolder
//the format is: ./WIMON_DD_MM_YY/
//the path to the folder can be obtained from the config file --implement this--
// only if logging is enabled

char time_string[15];
char folder[50];
struct timeval tv;
struct tm *ptm;
gettimeofday(&tv,NULL);	
ptm=localtime(&tv.tv_sec);
strftime(time_string,sizeof(time_string),"%d_%m_%y",ptm);
sprintf(folder,"./WiMon_%s",time_string);	
if (chdir(folder)!=0)
	{
	// the folder does not exist. create a new one.
	mkdir(folder,777);
	chdir(folder);	
	}

// no server test is used only when developing outside inria
// this means, no predictable APs are around	
#ifndef NO_SERVER_TEST

// connection to all kismet servers
for (server_counter=0;server_counter<kismet_servers.size();server_counter++)
	{
	while(open_kismet_client(&(kismet_servers[server_counter].filedes),kismet_servers[server_counter].socket,port,(char*)kismet_servers[server_counter].IP.c_str()))
		//&client_fd2,client_socket2,port,second_server) < 0)
		{
			if(count>10)
			{
				cout<< "Kismet" << "Can't connect to the server " << kismet_servers[server_counter].IP.c_str() <<"Quit" <<endl;
				exit(2);
			}
			sleep(1);
			count++;
		}
	}

// wait 10sec for the data from the server (the presentation stuff)

// serv_start receives the result value from the servers.
// if any of the servers answers, continues
// if none of the servers answer, it quits.



// check for server start message

for (server_counter=0;server_counter<kismet_servers.size();server_counter++)
	{
	count=0;	
	while(!(check_kismet_client(kismet_servers[server_counter].socket)))
		{
		count++;
		sleep(1);	
		}
	if(count>10)
		{
		cout<< "Kismet" << "Can't get start up data from the server " << kismet_servers[server_counter].IP.c_str() <<"Quit" <<endl;
		exit(2);
		}
	}

#endif


// here the connection to the kismet_server has already started.
// now, open the socket for listening
	
	
    /* Use default address: */

		srvr_addr = Laddress;
		char temp_port[40];	
	    sprintf(temp_port,"%d",Lport);
		srvr_port=temp_port;

    /*
     * Create a TCP/IP socket to use :
     */
    s = socket(PF_INET,SOCK_STREAM,0);
    if ( s == -1 )
        bail("socket()");
	
	/*
	 * Make it non-blocking
	 */
	fcntl(s,F_SETFL,O_NONBLOCK);
    	
	/*
	 * Enable the reuse feature in the socket
	 */
	 
	setsockopt(s,SOL_SOCKET,SO_REUSEADDR,&so_reuseaddr,sizeof(so_reuseaddr));
	
    /*
     * Create a server socket address:
     */
    memset(&adr_srvr,0,sizeof adr_srvr);
    adr_srvr.sin_family = AF_INET;
    adr_srvr.sin_port = htons(atoi(srvr_port));
    if ( strcmp(srvr_addr,"*") != 0 ) {
        /* Normal Address */
        adr_srvr.sin_addr.s_addr =
            inet_addr(srvr_addr);
        if ( adr_srvr.sin_addr.s_addr
             == INADDR_NONE )
            bail("bad address.");
    } else {
        /* Wild Address */
        adr_srvr.sin_addr.s_addr =
            INADDR_ANY;
    }

    /*
     * Bind the server address:
     */
    len_inet = sizeof adr_srvr;
    z = bind(s,(struct sockaddr *)&adr_srvr,
            len_inet);
    if ( z == -1 )
        bail("bind(2)");

    /*
     * Make it a listening socket:
     */
    z = listen(s,10);
    if ( z == -1 )
        bail("listen(2)");

#ifndef NO_SERVER_TEST

//start gathering packets
//enables packet transfer from every source.

// configure the features to transmit from the servers.	
strcpy(data,"!0 ENABLE PACKET type,subtype,timesec,timeusec,encrypted,weak,beaconrate,sourcemac,destmac,bssid,ssid,prototype,sourceip,destip,sourceport,destport,nbtype,nbsource,signal,retry,fullsize,datarate\n");
	
for (server_counter=0;server_counter<kismet_servers.size();server_counter++)
	{	
	send_kismet_client(kismet_servers[server_counter].socket,data);
	}
	
strcpy(data,"");	

#endif

// opens logging files
// missing error checking
	
packlog1=fopen ("logfile1.dump","w");
packlog2=fopen ("logfile2.dump","w");
mergepacklog=fopen ("logmerge.dump","w");

// kismet_multiple: call to the main routine
// loops:
//	accept connection
//  packet processing 
//  eventual disconnection and socket closing 
	
kismet_multiple
	(
	s,
	listpacket1,
	listpacket2,
	mergepacket,
	traffic_bin,
	packlog1,
	packlog2,
	mergepacklog,
	client_fd1,
	client_fd2,
	client_socket1,
	client_socket2,
	kismet_servers,
	logfiles
	);

// from this point on, the only thing remaining is to
// turn off the server.
// this is never reached now.
// a catchshutdown signal could help here, but the parameters should
// be global.
	
// save all the traffic information before dying

// close file descriptors

fclose (packlog1);
fclose (packlog2);
fclose (mergepacklog);

return 0;
}

// save packets:
// goes thru the list of all traffic information bins and saves it on a merge file.
// traffic: reference to the traffic instance of a map
// logfile: file descriptor of the file where to save it

void save_packets(map<unsigned long,unsigned int>& traffic,FILE* logfile)
{
	
	for (map<unsigned long,unsigned int>::iterator i=traffic.begin();
		i!=traffic.end();i++)
		{
		fprintf(logfile,"Bin: %ld Traffic Qty: %d\n",i->first,i->second);
		}
}	


void save_one_bin(map<unsigned long,unsigned int>::iterator i,FILE* logfile)
{

fprintf(logfile,"Bin: %ld Traffic Qty: %d\n",i->first,i->second);
		
}	

/************************************************************/

// kismet_multiple: logging engine
// gets the packets and process them to find stats.

void kismet_multiple(int s,
vector<packet_info>& listpacket1,
vector<packet_info>& listpacket2,
//list of merged packets
map<uint64_t,packet_info>& mergepacket,
// per second packet qty traffic information
map<unsigned long,unsigned int>& traffic_bin,
// file descriptors
FILE *packlog1,
FILE *packlog2,
FILE *mergepacklog,
// file descriptors to manage a socket as a file for fgets and fputs
FILE *client_fd1,
FILE *client_fd2,
// the sockets to manage connections
int& client_socket1,
int& client_socket2,
vector<kismet_server_struct>& kismet_servers,
map<mac_addr,logfile_structure>& logfiles
)

{

int server_counter;	
	
// these variables keeps track of the oldest time_bin.
 unsigned long int oldest_time_bin=0;
 unsigned long int oldest_traffic_bin=0;	

// string to receive the packet
char data[1024];	
	
// packet counter for grabbing data
int numpackserv1;
int numpackserv2;

// socket support
	socklen_t len_inet;               /* length  */
	int c;                			  /* Client socket */
	struct sockaddr_in adr_clnt;      /* AF_INET */
	FILE* fdclient; 				  /* Client stream */
// end socket support
	
	
// count: old variable, no use now, debugging and testing purposes.
int count=0;

// packet vectors:
// temporary reservoir of packets for each source
vector<packet_info> packet1;
vector<packet_info> packet2;

// this flag is set to 1 when the first socket connection is accepted.
// this should be removed when a multiple connection scheme is adopted.	

int socket_flag=0;

// strings that hold the protocol 
// the string class may help here. 
// this has been a source of segmentation faults in the past.

char request[500];
char parameter_request[200];
char mac_address_request[180];
char header_answer[200];
char data_answer [200]; //enough space for bin number,
						// comma separator,
						// data for bin,
						// space separator
char parameter[200];
char load_answer[10000]=""; // the the data section of the answer
char full_answer[10000]=""; // the complete answer
char start_bin[200];

// to collect the response from the polling	
int output1;
int output2;


// list of active sockets
fd_set all_socks_read,all_socks_read_bak;
fd_set all_socks_write,all_socks_write_bak;
fd_set all_socks_error,all_socks_error_bak;
fd_set new_socks;

FD_ZERO(&all_socks_read);
FD_ZERO(&all_socks_write);
FD_ZERO(&all_socks_error);
FD_ZERO(&new_socks);

int max_socket_number=0;

map <int,FILE*> socket_fd_list;
map <int,FILE*>::iterator conn_count;

unsigned long int last_update=0;

//the main processing loop starts here.

while (1) 	
	{
	// accept socket connection

	len_inet = sizeof adr_clnt;
	c = accept(s,
		(struct sockaddr *)&adr_clnt,
		&len_inet);	
	if (c!=-1)
		{	
		//we have a valid socket
		//open a stream associated with the socket
		fcntl(c,F_SETFL,O_NONBLOCK);	
		fdclient = fdopen(c, "r+");
		socket_flag=1;
		socket_fd_structure temp_socket_structure;
		temp_socket_structure.socket_connected=c;
		temp_socket_structure.filedes_connected=fdclient;
		socket_fd_list[c]=fdclient;	
		// add it to the socket set
		FD_SET(c,&new_socks);
		//keep the max assigned socket number
		if (c>max_socket_number)
			{
			max_socket_number=c;
			}
#ifdef DEBUG		

		printf("accepted connection: %d\n",c);
	
#endif
		}
	else
		{
#ifdef DEBUG
		// nothing else to do.	
		printf("waiting connection");
#endif			
		}	


			
	// check if there is a request waiting
	// a timeout checking for dead connections and
	// a multiple socket answering system is implemented. 		
    
	timeval tv;
	tv.tv_sec=0;
	tv.tv_usec=1;
		
	FD_ZERO(&all_socks_read);
	FD_ZERO(&all_socks_write);	
	
	//build the sets for select	
	for (int z=0;z<max_socket_number+1;z++)
		{
		if (FD_ISSET(z,&new_socks))
			{
			FD_SET(z,&all_socks_read);
			FD_SET(z,&all_socks_write);
			FD_SET(z,&all_socks_error);				
			}
		}
	// wait for 10 seconds until a packet arrives.	
	tv.tv_sec= 10;
	tv.tv_usec=1;

for (server_counter=0;server_counter<kismet_servers.size();server_counter++)
	{
    FD_SET(kismet_servers[server_counter].socket,&all_socks_read);		
	FD_SET(kismet_servers[server_counter].socket,&all_socks_write);
	if (kismet_servers[server_counter].socket>max_socket_number)
			{
			max_socket_number=kismet_servers[server_counter].socket;//keep the max assigned socket number
			}		
	}	

// get the maximum socket number waiting
		
	int n=select(max_socket_number+1,&all_socks_read,NULL,NULL,&tv);
	char linea[200];		

	for (conn_count=socket_fd_list.begin();conn_count!=socket_fd_list.end();conn_count++)	
		{
		
#ifdef DEBUG 
	printf("max_socket_number:%d\n",max_socket_number);
#endif			

#ifdef DEBUG 
	printf("n:%d\n",socket_fd_list.size());
#endif		

#ifdef DEBUG 
	printf("connection count:%d\n",conn_count->socket_connected);
#endif			
			
#ifdef DEBUG 
	printf("socket_connected:%d\n",socket_fd_list[connection_count].socket_connected);
#endif			

#ifdef DEBUG 
	printf("filedes_connected:%d\n",socket_fd_list[connection_count].filedes_connected);
#endif			

	if (FD_ISSET(conn_count->first,&all_socks_read))						
		{
#ifdef DEBUG		
			printf("request00: %s",request);
			printf("answering to socket: %d ",conn_count->first);
#endif	
			
// poll to see if the connection is still alive:
				
		char buffernew[32];
		if(recv(conn_count->first,buffernew,sizeof(buffernew),MSG_PEEK | MSG_DONTWAIT)==0)
			{			
			close(conn_count->first);
			socket_fd_list.erase(socket_fd_list.find(conn_count->first));
#ifdef DEBUG		
			printf("request00: %s",request);
			printf("closed socket: %d ",conn_count->first);
			
#endif	
			}				
		else
			{	
				fdclient=conn_count->second;	
				if (fgets(request,50,fdclient)!=NULL)
					{
#ifdef DEBUG		
					printf("request00: %s on socket: %d",request,conn_count->first);
					printf("sizeof request: %d ",sizeof(request));
#endif
				//	there is something to process
				//  is valid what we have? 18 is the minimum request.
					if (sizeof(request)>18)
						{
						sscanf (request,"%s %s %s", parameter,mac_address_request,start_bin);
#ifdef DEBUG		
						printf("parameter: %s mac_address_request: %s starting bin: %ld",parameter,mac_address_request,start_bin);
#endif				
						
#ifdef DEBUG		
						
						printf("strcmp: %d",strcmp(parameter,"PACKET_TRAFFIC"));
				
#endif				
							
						// clear answer 
						strcpy(full_answer,"\0");						
						strcpy(load_answer,"\0");		
							
// PACKET_TRAFFIC.
						
						// answer what it was requested
						if ((strcmp(parameter,"PACKET_TRAFFIC")==0))
							{
							sprintf(header_answer,"PACKET_TRAFFIC ",traffic_bin.size());
							// should check here for ability to send the info						
							strcat(full_answer,header_answer);
							unsigned int data_size=0;	
							// go thru all the list of available elements
							for (map<unsigned long,unsigned int>::iterator i=traffic_bin.begin();
							i!=traffic_bin.end();i++)
									{
									unsigned long int conv_start_bin;	
									sscanf(start_bin,"%d",&conv_start_bin);	
									if((i->first)>conv_start_bin)
										{
										data_size++;		
										sprintf(data_answer,"%ld,%d ",i->first,i->second);
#ifdef DEBUG		
										printf("%s",data_answer);
#endif				
										strcat(load_answer,data_answer);	
										}
									}
								if (data_size>0)
									{										
									sprintf(header_answer,"%d ",data_size);		
									strcat(full_answer,header_answer);
									strcat(full_answer,load_answer);												
									strcat(full_answer,"\n");
									}
								else
									{
									strcpy(full_answer,"ERROR\n");
									}
							}
						
// PACKET_TRAFFIC_OUT for a specific mac address.					
							
							if (strcmp(parameter,"PACKET_TRAFFIC_OUT")==0)
								{
								if (strlen(mac_address_request)==17)
									{
									mac_addr mac_temp;
									mac_temp.string2long(mac_address_request);								
									map<mac_addr,device_info>::iterator all_devices_index;	
									all_devices_index=all_devices.find(mac_temp);	
									if (all_devices_index!=all_devices.end())
										{
															
										sprintf(header_answer,"PACKET_TRAFFIC_OUT ");
										// should check here for ability to send the info						
										strcat(full_answer,header_answer);	
										unsigned int data_size=0;							
										// go thru all the list of available elements
										for (map<unsigned long int,bin_info>::iterator i=(all_devices_index->second).begin();
										i!=(all_devices_index->second).end();i++)
												{
												unsigned long int conv_start_bin;	
												sscanf(start_bin,"%d",&conv_start_bin);	
												if((i->first)>conv_start_bin)
													{
													data_size++;		
													sprintf(data_answer,"%ld,%d ",i->first,i->second.traffic_out_packets);
		#ifdef DEBUG		
													printf("%s",data_answer);
		#endif				
												strcat(load_answer,data_answer);	
												}
											}
									if (data_size>0)
										{										
										sprintf(header_answer,"%d ",data_size);		
										strcat(full_answer,header_answer);
										strcat(full_answer,load_answer);												
										strcat(full_answer,"\n");
										}
									else
										{
										strcpy(full_answer,"ERROR\n");
										}			
									}
									else
									{
										sprintf(data_answer,"ERROR\n");
										strcat(full_answer,data_answer);
									}
								}
							}
							
// PACKET_TRAFFIC_IN for a specific mac_address.					
						
							if (strcmp(parameter,"PACKET_TRAFFIC_IN")==0)
								{
								if (strlen(mac_address_request)==17)
									{
									mac_addr mac_temp;
									mac_temp.string2long(mac_address_request);								
									map<mac_addr,device_info>::iterator all_devices_index;	
									all_devices_index=all_devices.find(mac_temp);	
									if (all_devices_index!=all_devices.end())
										{
															
										sprintf(header_answer,"PACKET_TRAFFIC_IN ");
										// should check here for ability to send the info						
										strcat(full_answer,header_answer);		
										unsigned int data_size=0;
										// go thru all the list of available elements
										for (map<unsigned long int,bin_info>::iterator i=(all_devices_index->second).begin();
										i!=(all_devices_index->second).end();i++)
												{
												unsigned long int conv_start_bin;	
												sscanf(start_bin,"%d",&conv_start_bin);	
												if((i->first)>conv_start_bin)
													{
													data_size++;	
													sprintf(data_answer,"%ld,%d ",i->first,i->second.traffic_in_packets);
		#ifdef DEBUG		
													printf("%s",data_answer);
		#endif				
													strcat(load_answer,data_answer);		
													}	
												}
									if (data_size>0)
										{										
										sprintf(header_answer,"%d ",data_size);		
										strcat(full_answer,header_answer);
										strcat(full_answer,load_answer);												
										strcat(full_answer,"\n");
										}
									else
										{
										strcpy(full_answer,"ERROR\n");
										}											
									}
									else
									{
										sprintf(data_answer,"ERROR\n");
										strcat(full_answer,data_answer);	
									}
								}
							}
						
// BIT_TRAFFIC_IN for a specific mac_address.					
						
							if (strcmp(parameter,"BIT_TRAFFIC_IN")==0)
								{
								if (strlen(mac_address_request)==17)
									{
									mac_addr mac_temp;
									mac_temp.string2long(mac_address_request);								
									map<mac_addr,device_info>::iterator all_devices_index;	
									all_devices_index=all_devices.find(mac_temp);	
									if (all_devices_index!=all_devices.end())
										{
										sprintf(header_answer,"BIT_TRAFFIC_IN ");
										// should check here for ability to send the info						
										strcat(full_answer,header_answer);		
										unsigned int data_size=0;							
										// go thru all the list of available elements
										for (map<unsigned long int,bin_info>::iterator i=(all_devices_index->second).begin();
										i!=(all_devices_index->second).end();i++)
												{
												unsigned long int conv_start_bin;	
												sscanf(start_bin,"%d",&conv_start_bin);	
												if((i->first)>conv_start_bin)
													{
													data_size++;			
													sprintf(data_answer,"%ld,%d ",i->first,i->second.traffic_in_bits);
		#ifdef DEBUG		
												printf("%s",data_answer);
		#endif				
												strcat(load_answer,data_answer);	
												}
											}

										if (data_size>0)
										{										
										sprintf(header_answer,"%d ",data_size);		
										strcat(full_answer,header_answer);
										strcat(full_answer,load_answer);												
										strcat(full_answer,"\n");
										}
									else
										{
										strcpy(full_answer,"ERROR\n");
										}
									}
									else
									{
										sprintf(data_answer,"ERROR\n");
										strcat(full_answer,data_answer);
									}
								}
							}

// BIT_TRAFFIC_OUT for a specific mac_address.					
						
							if (strcmp(parameter,"BIT_TRAFFIC_OUT")==0)
								{
								if (strlen(mac_address_request)==17)
									{
									mac_addr mac_temp;
									mac_temp.string2long(mac_address_request);								
									map<mac_addr,device_info>::iterator all_devices_index;	
									all_devices_index=all_devices.find(mac_temp);	
									if (all_devices_index!=all_devices.end())
										{
										sprintf(header_answer,"BIT_TRAFFIC_OUT ");
										// should check here for ability to send the info						
										strcat(full_answer,header_answer);
										unsigned int data_size=0;	
										// go thru all the list of available elements
										for (map<unsigned long int,bin_info>::iterator i=(all_devices_index->second).begin();
										i!=(all_devices_index->second).end();i++)
												{
												unsigned long int conv_start_bin;	
												sscanf(start_bin,"%d",&conv_start_bin);	
												if((i->first)>conv_start_bin)
													{
													data_size++;			
													sprintf(data_answer,"%ld,%d ",i->first,i->second.traffic_out_bits);
		#ifdef DEBUG		
												printf("%s",data_answer);
		#endif				
												strcat(load_answer,data_answer);	
												}
											}	
										if (data_size>0)
										{										
										sprintf(header_answer,"%d ",data_size);		
										strcat(full_answer,header_answer);
										strcat(full_answer,load_answer);												
										strcat(full_answer,"\n");
										}
									else
										{
										strcpy(full_answer,"ERROR\n");
										}
									}
									else
									{
										sprintf(data_answer,"ERROR\n");
										strcat(full_answer,data_answer);
									}
								}
							}
							
		
// POWER for a specific mac_address.					
					
							if (strcmp(parameter,"POWER")==0)
								{
								if (strlen(mac_address_request)==17)
									{
									mac_addr mac_temp;
									mac_temp.string2long(mac_address_request);								
									map<mac_addr,device_info>::iterator all_devices_index;	
									all_devices_index=all_devices.find(mac_temp);	
									if (all_devices_index!=all_devices.end())
										{
															
										sprintf(header_answer,"POWER ");
										// should check here for ability to send the info						
										strcat(full_answer,header_answer);	
										unsigned int data_size=0;							
										// go thru all the list of available elements
										for (map<unsigned long int,bin_info>::iterator i=(all_devices_index->second).begin();
										i!=(all_devices_index->second).end();i++)
												{
												unsigned long int conv_start_bin;	
												sscanf(start_bin,"%d",&conv_start_bin);	
												if((i->first)>conv_start_bin)
													{
													data_size++;			
													int power_level;
													if(i->second.traffic_out_packets==0)
														{
														power_level=i->second.power;
														}
													else
														{
														power_level=(i->second.power/i->second.traffic_out_packets);
														}
													sprintf(data_answer,"%ld,%d ",i->first,power_level);
		#ifdef DEBUG		
												printf("%s",data_answer);
		#endif				
												strcat(load_answer,data_answer);	
												}
											}	
										if (data_size>0)
										{										
										sprintf(header_answer,"%d ",data_size);		
										strcat(full_answer,header_answer);
										strcat(full_answer,load_answer);												
										strcat(full_answer,"\n");
										}
									else
										{
										strcpy(full_answer,"ERROR\n");
										}
									}
									else
									{
										sprintf(data_answer,"ERROR\n");
										strcat(full_answer,data_answer);	
									}
								}
							}
							
// CONNECTION_SPEED for a specific mac_address.					
						
							if (strcmp(parameter,"CONNECTION_SPEED")==0)
								{
								if (strlen(mac_address_request)==17)
									{
									mac_addr mac_temp;
									mac_temp.string2long(mac_address_request);								
									map<mac_addr,device_info>::iterator all_devices_index;	
									all_devices_index=all_devices.find(mac_temp);	
									if (all_devices_index!=all_devices.end())
										{
															
										sprintf(header_answer,"CONNECTION_SPEED ");
										// should check here for ability to send the info						
										strcat(full_answer,header_answer);	
										unsigned int data_size=0;							
										// go thru all the list of available elements
										for (map<unsigned long int,bin_info>::iterator i=(all_devices_index->second).begin();
										i!=(all_devices_index->second).end();i++)
												{
												unsigned long int conv_start_bin;	
												sscanf(start_bin,"%d",&conv_start_bin);	
												if((i->first)>conv_start_bin)
													{
													data_size++;			
													sprintf(data_answer,"%ld,%d ",i->first,i->second.bandwidth);	
		#ifdef DEBUG		
												printf("%s",data_answer);
		#endif				
												strcat(load_answer,data_answer);	
												}
											}	
										if (data_size>0)
										{										
										sprintf(header_answer,"%d ",data_size);		
										strcat(full_answer,header_answer);
										strcat(full_answer,load_answer);												
										strcat(full_answer,"\n");
										}
									else
										{
										strcpy(full_answer,"ERROR\n");
										}
									}
									else
									{
										sprintf(data_answer,"ERROR\n");
										strcat(full_answer,data_answer);
									}
								}
							}
							
// RETRY_TIMES for a specific mac_address.					
							
						
							if (strcmp(parameter,"RETRY_TIMES")==0)
								{
								if (strlen(mac_address_request)==17)
									{
									mac_addr mac_temp;
									mac_temp.string2long(mac_address_request);								
									map<mac_addr,device_info>::iterator all_devices_index;	
									all_devices_index=all_devices.find(mac_temp);	
									if (all_devices_index!=all_devices.end())
										{
															
										sprintf(header_answer,"RETRY_TIMES ");
										// should check here for ability to send the info						
										strcat(full_answer,header_answer);	
										unsigned int data_size=0;							
										// go thru all the list of available elements
										for (map<unsigned long int,bin_info>::iterator i=(all_devices_index->second).begin();
										i!=(all_devices_index->second).end();i++)
												{
												unsigned long int conv_start_bin;	
												sscanf(start_bin,"%d",&conv_start_bin);	
												if((i->first)>conv_start_bin)
													{
													data_size++;			
													sprintf(data_answer,"%ld,%d ",i->first,i->second.retransmission);
		#ifdef DEBUG		
												printf("%s",data_answer);
		#endif				
												strcat(load_answer,data_answer);	
												}
											}	
										if (data_size>0)
										{										
										sprintf(header_answer,"%d ",data_size);		
										strcat(full_answer,header_answer);
										strcat(full_answer,load_answer);												
										strcat(full_answer,"\n");
										}
									else
										{
										strcpy(full_answer,"ERROR\n");
										}
									}
									else
									{
										sprintf(data_answer,"ERROR\n");
										strcat(full_answer,data_answer);
									}
								}
							}

// LAST for a specific mac_address.					
						
							if (strcmp(parameter,"LAST")==0)
								{
								if (strlen(mac_address_request)==17)
									{
									mac_addr mac_temp;
									mac_temp.string2long(mac_address_request);								
									map<mac_addr,device_info>::iterator all_devices_index;	
									all_devices_index=all_devices.find(mac_temp);	
									if (all_devices_index!=all_devices.end())
										{
										sprintf(header_answer,"LAST %d ",8);
										// should check here for ability to send the info						
										strcat(full_answer,header_answer);	
										// go thru all the list of available elements
										map<unsigned long int,bin_info>::iterator p=(all_devices_index->second).end();
										if ((all_devices_index->second).size()!=0)
												{
												//lets look at the last time bin:	
												p--;p--;
												int power_level;
												if(p->second.traffic_out_packets==0)
													{
													power_level=p->second.power;
													}
												else
													{
													power_level=(p->second.power/p->second.traffic_out_packets);
													}	
												sprintf(data_answer,"%d %d %d %d %d %d %d %d",
													p->second.traffic_in_bits,
													p->second.traffic_in_packets,
													p->second.traffic_out_bits,
													p->second.traffic_out_packets,
													power_level,
													p->second.retransmission,
													p->second.mode,
													p->second.bandwidth);
		#ifdef DEBUG		
												printf("salida %s",data_answer);
		#endif				
												strcat(full_answer,data_answer);	
												}		
										strcat(full_answer,"\n");		
									}
									else
									{
										sprintf(data_answer,"ERROR\n");
										strcat(full_answer,data_answer);
									}
								}
							}
						}
					//here the response (saved on full_answer) is sent, after checking
					//one-second timeout - if not available to write, ignore request.
					//i shall test here for communication errors.				
					tv.tv_sec=1;
					tv.tv_usec=0;
					FD_ZERO(&all_socks_write);
					FD_ZERO(&all_socks_read);	
					FD_SET(conn_count->first,&all_socks_read);		
					FD_SET(conn_count->first,&all_socks_write);						
					select((conn_count->first)+1,NULL,&all_socks_write,NULL,&tv);					
					if (FD_ISSET(conn_count->first,&all_socks_write))	
						{
			#ifdef DEBUG		
						int tt=conn_count->first;
						printf("fdclient answered %d",conn_count->first);
			#endif							
						fdclient=conn_count->second;	
						write(conn_count->first, full_answer, strlen(full_answer));							
						}
					}
				else
					{
					if (errno != 0 && errno != EAGAIN) 
						{
						printf("Read error %d (%s), closing the connection.",errno, strerror(errno));
						close(conn_count->first);	
						//erase the socket from the list
						socket_fd_list.erase(socket_fd_list.find(conn_count->first));	
						}
					}
				}
			}

		}
	
#ifndef NO_SERVER_TEST
		
	// both servers must have answered before going on.
	// timeout is recommended. Packet qty can be asked before grabbing.
	// of course, if timeout, packet loss will happen.
	// to improve log traffic numbers, less parameters should be asked for. 

for (server_counter=0;server_counter<kismet_servers.size();server_counter++)
	{
	int ki=0;	
	while (ki<100)
	    {
	    ki++;
		output1=receive_kismet_client(kismet_servers[server_counter].socket,kismet_servers[server_counter].filedes,data);
		if (output1<0)
			{
			printf("Kismet server closed connection:Closing");	
			exit(output1);
			}
		if (strcmp(data,"")!=0)
			{
			load_kismet_client(data,packet1);
			}
		strcpy(data,"");	

#ifdef DEBUG_R
	    printf("ki1:%d\t",ki);
#endif
	    }
		
		{

		// is there anything on packet1 list?
		if (packet1.size()>0)
			{
#ifdef DEBUG_R
			printf("*******packet1 size: %d \n",packet1.size());			
#endif			
			// pack: travels thru the whole packet list.
			vector<packet_info>::iterator pack = packet1.begin();
			while( pack != packet1.end()) 
				{
				//merge them in the mergepacklist
				packinsert(mergepacket,*pack);
				//printf("mergepacket time stamp high: %ld mergepacket time stamp low: %ld \n",pack->ts.tv_sec,pack->ts.tv_usec);	
				//debug to find out the size
				int j=mergepacket.size();
				// go to next register on the packet list
				pack++;
				}
			// erase the current list of packets to continue.	
			packet1.clear();	
#ifdef DEBUG_R
			printf("listpacket1 size: %d \n",listpacket1.size());
			printf("mergepacket size: %d \n",mergepacket.size());
#endif
			}		

		}
	}	

		
	//save and process packets
	//traffic measurement method:
	//build time bins where packet quantity is saved for every second.
	//the bins structure has a maximum size, and when this limit
	//is reached, if there is a new bin waiting, it drops the oldest.
	//if there is no packet for the bin, it is created, with 0 value.
	//it only builds bins where there are spackets.
	//the bin quantity is based on the current buffer information.
	// when there is enough information gathered, process it to find the traffic
	// 100 packets is a good number.
			
		{
		// merge goes thru the whole merged packet list.
		map<uint64_t,packet_info>::iterator merge = mergepacket.begin();
		// index_bin points to a certain time bin where packets are accumulated
		map<unsigned long,unsigned int>::iterator index_bin;
		
		while( merge != mergepacket.end())
			{
#ifdef DEBUG_R
		printf("fullsize: %x",merge->second.fullsize);
#endif				
			// traffic bin load and processing
			// first record in traffic bin
			// intialize it.
			if (traffic_bin.size()==0)
				{
				// true: this is the first packet.
				unsigned int pack_qty=1;
				unsigned long seconds;
				seconds=merge->second.ts.tv_sec;
				// create the first bin.
				traffic_bin[seconds]=pack_qty*merge->second.fullsize*8;
				}
			else
				{
				// it is not empty, then try to find out
				// if there is a bin which fits.
				// is there any bin where this packet fits?
				// flag is used to insure we get out of the do-while statement			
				int flag=0;
				// start from the beggining
				index_bin=traffic_bin.begin();
				index_bin=traffic_bin.find(merge->second.ts.tv_sec);
				if (index_bin!=traffic_bin.end())
					{
					// does the current bin correspond to the
					// packet?
						{
						index_bin->second+=merge->second.fullsize*8;
						//ok, the current packet fits there, no more comparisons.
						flag=1;
						}
					// do not increase the pointer further than the end of the map.
					// find would do a better job, but by now this works ok.
					//index_bin++;	
					}
				// if the following is true
				// we did not find a bin for this packet.
				// this means, to this point, flag=0.
				// we reached the end of the list with
				// no traffic bin found to increase,
				// then a new bin must be added
				if (flag==0)
					{
					// add one to the corresponding packet
					unsigned int pack_qty=1;
					unsigned long seconds;
					seconds=merge->second.ts.tv_sec;
					//create a new bin.
					traffic_bin[seconds]=pack_qty*merge->second.fullsize*8; //measured in bits per second
					// check if there are too many bins
					if (traffic_bin.size()>TRAFFIC_BIN_MAX_SIZE)
					    {
						// find out which to erase (the oldest, of course)	
						map<unsigned long,unsigned int>::iterator min_bin,min_time_it;
						unsigned long int min_time=traffic_bin.begin()->first;
						for (min_bin=traffic_bin.begin();min_bin!=traffic_bin.end();min_bin++)
							if (min_time>min_bin->first)
								{
								min_time=min_bin->first;
								//min_time_it=min_bin;
								}
						//save_one_bin(min_time_it,mergepacklog);		
					    traffic_bin.erase(traffic_bin.find(min_time));
					    }				
					}
				}

			map<mac_addr,device_info>::iterator all_devices_index;	
			if (all_devices.size()==0)
				{
				// true: this is the first device.
				// devices are classified by source address.
				// so the first packet of the first device 
				// will have no input traffic.
				bin_info temp_info;	
				temp_info.traffic_in_bits=0;
				temp_info.traffic_in_packets=0;
				//must add caplen to the packet_info structure from the server
				//thru all the communication to the client.
	 			temp_info.traffic_out_bits=(merge->second.fullsize)*8;
	 			temp_info.traffic_out_packets=1;
				temp_info.power=merge->second.signal;
		// testing the offset:
				//must add retransmission bit in the structure of packet_info					
				//thru all the communication to the client.
				temp_info.retransmission=0;
				temp_info.mode=merge->second.maxrate;
				temp_info.bandwidth=merge->second.datarate;
				temp_info.saved=0;				
				// create the first device.
				map<unsigned long int,bin_info> temp_map;
				temp_map[merge->second.ts.tv_sec]=temp_info;
				all_devices[merge->second.source_mac]=temp_map;
				}
			else
				{
				// it is not empty, then try to find out
				// if there is a device which fits.
				// is there any device where this packet fits?
				// flag is used to insure we get out of the do-while statement
				int flaginput=0;					
				int flagoutput=0;
				// start from the beggining
				all_devices_index=all_devices.begin();
				if (all_devices.find(merge->second.source_mac)!=all_devices.end())			
					{
					all_devices_index=all_devices.find(merge->second.source_mac);	
					// here there is an interesting problem:
					// for each device, i must check if there is a time_bin that
					// fits the current packet.
					// if there is no device, create a new device.
					// if there is no time_bin for a specific device, create it.						
#ifdef DEBUG_R
					string temp_str;
					temp_str=all_devices_index->first.Mac2String();		
					
#endif						
						{
						// flag is used to insure we get out of the do-while statement			
						int flagin=0;
						map<unsigned long int,bin_info>::iterator all_timebins_index;		
						// start from the beggining
						if ((all_devices_index->second).find(merge->second.ts.tv_sec)!=(all_devices_index->second).end())
							{
							unsigned long int g=all_timebins_index->first;
							all_timebins_index=(all_devices_index->second).find(merge->second.ts.tv_sec);								
								{
								// this is only transmission of packets	
								//must add caplen to the packet_info structure from the server
								//thru all the communication to the client.
								all_timebins_index->second.traffic_out_bits+=(merge->second.fullsize)*8;//caplen
								all_timebins_index->second.traffic_out_packets++;
								//a power mean value must be taken into account, so the number will be
								//the cumulative power values and should be divided by the number of packets
								//in the current bin.
						// testing the offset			
								all_timebins_index->second.power+=merge->second.signal;
								//if ((merge->second.signal && 0x8000)>0)
								//	{
								//	all_timebins_index->second.power+=(~(merge->second.signal))+1;
								//	}
								//else
								//	{					
								//	all_timebins_index->second.power+=merge->second.signal;
								//	}
								
					
								//must add retransmission bit in the structure of packet_info					
								//thru all the communication to the client.
								all_timebins_index->second.retransmission+=merge->second.retry;
								//mode will keep the last value of max rate in the bin					
								all_timebins_index->second.mode=merge->second.maxrate;
								//bandwidth will keep the cumulative bandwidth on the whole bin.
								//this value must be divided by the packet_qty to get a mean value.
								all_timebins_index->second.bandwidth=merge->second.datarate;		
								//for power calculations and for bandwidth calculations the mean value
								//calculated on the packet qty may not be valid if we think that the packets
								//have different lengths.
								//ok, the current packet fits there, no more comparisons.
								flagin=1;
								}
								// do not increase the pointer further than the end of the map.
							}
						// we did not find a time_bin suitable for this packet. Let's create a new one.				
						else
							{
							// true: this is a new time_bin.
							// so we process the packet to get statistics for the new time_bin 
							// we will have no input traffic.
							bin_info temp_info;	
							temp_info.traffic_in_bits=0;
							temp_info.traffic_in_packets=0;
							//must add caplen to the packet_info structure from the server
							//thru all the communication to the client.
							temp_info.traffic_out_bits=(merge->second.fullsize)*8;
							temp_info.traffic_out_packets=1;
					
							temp_info.power=merge->second.signal;
							//must add retransmission bit in the structure of packet_info					
							//thru all the communication to the client.
							temp_info.retransmission=merge->second.retry;
							temp_info.mode=merge->second.maxrate;
							temp_info.bandwidth=merge->second.datarate;
							temp_info.saved=0;							
							// create the new time_bin.
							all_devices_index->second[merge->second.ts.tv_sec]=temp_info; 
#ifdef DEBUG
							printf("added: %ld \n",merge->second.ts.tv_sec);
#endif
							}
							if (all_devices_index->second.size()>TRAFFIC_BIN_MAX_SIZE)
								{
								// find out which to erase (the oldest, of course)	
								map<unsigned long int,bin_info>::iterator min_bin,min_time_it;
								unsigned long int min_time=(all_devices_index->second.begin())->first;
								for (min_bin=(all_devices_index->second).begin();min_bin!=(all_devices_index->second).end();min_bin++)
									{
					
									if ((min_bin->first)<min_time)
										{
										min_time=(min_bin->first);
										}
									}
								//remember to save information afterwards! we are loosing statistics.	
								all_devices_index->second.erase(all_devices_index->second.find(min_time));		
#ifdef DEBUG									
								printf("erased: %ld \n",min_time);
#endif									
								}
							
							//ok, the current packet fits there, no more comparisons.
							flagoutput=1;  // this flag is used to signal the end of the search loop, and
							};
							
// Received packet and traffic count processing				
	
// only after originating a packet, a new device is created. For each packet, after increasing
// the originator, we try to find out the destination mac address between the different nodes.
// of course, if the destination is not between the pre-existant nodes, no traffic will be
// added.
// we check only the destination mac, not the bssid mac.				
							
					if (all_devices.find(merge->second.dest_mac)!=all_devices.end())
						{
						// flag is used to insure we get out of the do-while statement			
						int flagin=0;
						map<unsigned long int,bin_info>::iterator all_timebins_index;		
						// start from the beggining
						all_timebins_index=(all_devices_index->second).begin();
						if ((all_devices_index->second).find(merge->second.ts.tv_sec)!=(all_devices_index->second).end())	
							{			
							all_timebins_index=(all_devices_index->second).find(merge->second.ts.tv_sec);
								{
								// this is only transmission of packets	
								all_timebins_index->second.traffic_in_bits+=(merge->second.fullsize)*8; //caplen
								all_timebins_index->second.traffic_in_packets++;
								//must add caplen to the packet_info structure from the server
								//thru all the communication to the client.
								//ok, the current packet fits there, no more comparisons.
								flagin=1;
								}
							// do not increase the pointer further than the end of the map.
							// find would do a better job, but by now this works ok.
							}
						// true: this is a new time_bin.
						// so we process the packet to get statistics for the new time_bin 
						// we did not find a time_bin suitable for this packet. Let's create a new one.				
						if (flagin==0)
								{
				// we initialize every parameter to 0 except the input_traffic.		
								bin_info temp_info;	
								temp_info.traffic_in_bits=(merge->second.fullsize)*8;
								temp_info.traffic_in_packets=1;
				//must add caplen to the packet_info structure from the server
				//thru all the communication to the client.
								temp_info.traffic_out_bits=0;
								temp_info.traffic_out_packets=0;
								temp_info.power=0;
				//must add retransmission bit in the structure of packet_info					
				//thru all the communication to the client.
								temp_info.retransmission=0;
								temp_info.mode=0;
								temp_info.bandwidth=0;
								temp_info.saved=0;									
				// create the new time_bin.
								all_devices_index->second[merge->second.ts.tv_sec]=temp_info; 
								}
						if (all_devices_index->second.size()>TRAFFIC_BIN_MAX_SIZE)
								{
								// find out which to erase (the oldest, of course)	
								map<unsigned long int,bin_info>::iterator min_bin,min_time_it;
								unsigned long int min_time=all_devices_index->second.begin()->first;
								for (min_bin=all_devices_index->second.begin();min_bin!=all_devices_index->second.end();min_bin++)
									if (min_time<min_bin->first)
										{
										min_time=min_bin->first;
										min_time_it=min_bin;
										}
									
								all_devices_index->second.erase(all_devices_index->second.find(min_time));
								}
							
								
							//ok, the current packet fits there, no more comparisons.
							flaginput=1; //there is a flag for incoming packets too. 
							};
					// do not increase the pointer further than the end of the map.
					// find would do a better job, but by now this works ok.
					// so, only if we have not found the device and we have not reached the end of the list			
					 }
				// if the following is true
				// we did not find a bin for this packet.
				// this means, to this point, flag=0.
				// we reached the end of the list with
				// no traffic output time bin found to increase,
				// then a new bin must be added
				else
					{
					// true: this is a new device.
					// devices are classified by source address.
					// so the first packet of the first device 
					// will have no input traffic.
					bin_info temp_info;	
					temp_info.traffic_in_bits=0;
					temp_info.traffic_in_packets=0;
					//must add caplen to the packet_info structure from the server
					//thru all the communication to the client.
					temp_info.traffic_out_bits=merge->second.fullsize;
					temp_info.traffic_out_packets=1;
					temp_info.power=merge->second.signal;
					//must add retransmission bit in the structure of packet_info					
					//thru all the communication to the client.
					temp_info.retransmission=merge->second.retry;
					temp_info.mode=merge->second.maxrate;
					temp_info.bandwidth=merge->second.datarate;
					temp_info.saved=0;					
					// create the new device.
					map<unsigned long int,bin_info> temp_map;
					temp_map[merge->second.ts.tv_sec]=temp_info;
					all_devices[merge->second.dest_mac]=temp_map;
					}
				}
#ifdef DEBUG_R
			printf("traffic_bin size: %d",traffic_bin.size());
#endif		
			
			// periodic logging every ten seconds
			if (logfil)
				if ((merge->second.ts.tv_sec-last_update)>10) // here we check the logging period when to save the
															// info. This means saving the last time we logged info
															// and then compare it with the current timestamp.
					{
					//here, go thru all the devices
					map<mac_addr,device_info>::iterator all_devices_index;
					for (all_devices_index=all_devices.begin();all_devices_index!=all_devices.end();all_devices_index++)
						{
						if (logfiles.find(all_devices_index->first)==logfiles.end()) // the logging file does not exist, then create only the file
							// then, the next condition will register the values.
							{
							//create new file	
							logfile_structure temp_logfile;
							strcpy(temp_logfile.file_name,all_devices_index->first.Mac2String().c_str());							
							temp_logfile.filedes_connected=fopen(temp_logfile.file_name,"a");
							temp_logfile.last_saved_bin=0;	
							logfiles[all_devices_index->first]=temp_logfile;
							}
						//now save the log rows.	
						map<unsigned long int,bin_info>::iterator index_bin;
						map<mac_addr,logfile_structure>:: iterator log_iterator=logfiles.find(all_devices_index->first);
						unsigned long int last_saved_bin_temp=log_iterator->second.last_saved_bin;	
						for (index_bin=all_devices_index->second.begin();index_bin!=all_devices_index->second.end();index_bin++)
							{
							//after the last saved, before the current bin.	
							
							if (index_bin->second.saved==0)								
								{
								int power_level;
								if(index_bin->second.traffic_out_packets==0)
									{
									power_level=index_bin->second.power;
									}
								else
									{
									power_level=(index_bin->second.power/index_bin->second.traffic_out_packets);
									}
								long int traffic_qty=0;	
								if (traffic_bin.find(index_bin->first)!=traffic_bin.end())
									{
									traffic_qty=traffic_bin.find(index_bin->first)->second;
									}										
								//here, the file exists, so we append the info.	
								fprintf(log_iterator->second.filedes_connected,"%ld,%d,%d,%d,%d,%d,%d,%d,%ld\n",
									index_bin->first,
									index_bin->second.traffic_in_bits,
									index_bin->second.traffic_in_packets,
									index_bin->second.traffic_out_bits,
									index_bin->second.traffic_out_packets,
									power_level,
									index_bin->second.retransmission,
									index_bin->second.bandwidth,
									traffic_qty);
								//put the info on the disk to check how it works. Not very efficient.	
								fflush(log_iterator->second.filedes_connected);
								
								index_bin->second.saved=1;	
								//update the last saved timestamp
								if (last_saved_bin_temp<index_bin->first)
									{									
									last_saved_bin_temp=index_bin->first;
									}									
								}									
							//update the last saved timestamp
							}
						//update the last saved timestamp	
						log_iterator->second.last_saved_bin=last_saved_bin_temp;
						if 	(last_update<last_saved_bin_temp)
							{
							last_update=last_saved_bin_temp;
							}								
						}
					
					}	
			// increase merging index.
			merge++;			
			}
		// now that we have processed all the information, we may clear all the intermediate buffers.
		mergepacket.clear();
		listpacket1.clear();
		listpacket2.clear();
		}
#endif //end of the NO_SERVER_TEST condition	
	}
		
}



// packet equal mine: this function keeps the extra rules to see if the inserting packet
// is the same as the packet inside the window.	 Not used now.
// it must be noted that there most common thing to be used here is the fcs at the end of
// the packet.
// now, only the time window check is used.
	
int packet_equal_mine(map<uint64_t,packet_info>::iterator search_index, packet_info &packtoinsert)
{
// criteria: one packet is repeated when it has the same sequence number
// within the timestamp error window.
// many other considerations can be taken tough, increasing used processing power.
// this function returns false if the sequence numbers are different
if (search_index->second.sequence_number==packtoinsert.sequence_number)
	{
	return (1);
	}
else
	{
	return (0);
	}
}

// packet insertion routine: it is supposed to add packets to the packet merge map.

void packinsert(map<uint64_t,packet_info>& mergepacket_ins, packet_info &packtoinsert)
{
// index and index_cur are used in intermediate calculations of map index.
uint64_t index,index_cur;
// search_ go thru the whole list of packets.
map<uint64_t,packet_info>::iterator search_start;
map<uint64_t,packet_info>::iterator search_end;
map<uint64_t,packet_info>::iterator search_index;
// dummy comparator is used for temporal comparisons.
packet_info dummy_comparator;
// we build one unique timestamp from the seconds and microseconds fields.
index=((uint64_t)(packtoinsert.ts.tv_sec))*1000000+((uint64_t)(packtoinsert.ts.tv_usec));
// debugging
int i=mergepacket_ins.size();
// insertion routine
if (mergepacket_ins.size()==0)
	{
	//first packet
	mergepacket_ins[index]=packtoinsert;
	// debugging
	int i=mergepacket_ins.size();
	}
else
	{
	//second packet we need at least two in the comparison window
	//for this algorithm to work.
	search_index=mergepacket_ins.lower_bound(index-WINDOWHALF);
	search_end=mergepacket_ins.upper_bound(index+WINDOWHALF);
	index_cur=((uint64_t)(search_index->second.ts.tv_sec))*1000000+((uint64_t)(search_index->second.ts.tv_usec));
	int comparisons=0;	
	do
		{
		//calculate current index:
		index_cur=((uint64_t)(search_index->second.ts.tv_sec))*1000000+((uint64_t)(search_index->second.ts.tv_usec));
		//test if the packet is within the pre-established window
		if (abs(((long long) (index-index_cur)))<WINDOWHALF)
		// go to the second test if it is in the window - deactivated
			{
			break;
			}
		comparisons++;	
		if (search_index!=search_end)
			// are we at the end of the map? do not increment the pointer.
			search_index++;
		}
	while (search_index!=search_end);
	// if we reached the end of the map, we should insert the packet.
	// the break effect forces the do-while statement to exit before
	// reaching the end of the list.
	if (search_index==search_end)
		{
		mergepacket_ins[index]=packtoinsert;
		}
	}
}
